#! /usr/bin/python
from mpi4py import MPI
from twisted.spread import pb

from ctlib.lib import wrapper

comm = MPI.COMM_WORLD

#This is the important class
class SPortfolio(pb.Referenceable):
    def __init__(self,derivatives):
        self.derivatives = derivatives
        self.rank = comm.Get_rank()
        self.size = comm.Get_size()
        print "instantiated",self.rank
        #put parallelism here

    def value_derivative(self,der):
        if der[0] == "OptionCallEuropeanSingle":
            op = wrapper.OptionCallEuropeanSingle(der[1],der[2],der[3],
                                          der[4],der[5],der[6])
            return op.value()
        
        if der[0] == "OptionPutEuropeanSingle":
            op = wrapper.OptionPutEuropeanSingle(der[1],der[2],der[3],
                                         der[4],der[5],der[6])
            return op.value()

        if der[0] == "OptionCallAmericanSingle":
            op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                          der[4],der[5],der[6])
            return op.value()

        if der[0] == "OptionPutAmericanSingle":
            op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                          der[4],der[5],der[6])
            return op.value()

    def spread_derivatives(self):
        pass
        

    def remote_value(self):
        if self.rank == 0:
            value = list() #for a gather.

        localvalue = 0.0

        print "rank",self.rank,self.derivatives
        
        for der in self.derivatives:
            #Compute the value
            localvalue += self.value_derivative(der)
            #Say potato
            print "Computed a " + der[0] + " value, node: ",self.rank

        return localvalue #temporary

    def remote_delta(self):
        delta = list()
        for der in self.derivatives:
            if der[0] == "OptionCallEuropeanSingle":
                op = wrapper.OptionCallEuropeanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionPutEuropeanSingle":
                op = wrapper.OptionPutEuropeanSingle(der[1],der[2],der[3],
                                             der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionCallAmericanSingle":
                op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                delta.append(op.delta())

            if der[0] == "OptionCallAmericanSingle":
                op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                delta.append(op.delta())

            print "Computed a " + der[0] + " delta"

        return delta

    def remote_gamma(self):
        gamma = 0.0

        for der in self.derivatives:
            if der(0) == "OptionCallEuropeanSingle":
                op = wrapper.OptionCallEuropeanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionPutEuropeanSingle":
                op = wrapper.OptionPutEuropeanSingle(der[1],der[2],der[3],
                                             der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionCallAmericanSingle":
                op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                gamma += op.gamma()

            if der(0) == "OptionCallAmericanSingle":
                op = wrapper.OptionCallAmericanSingle(der[1],der[2],der[3],
                                              der[4],der[5],der[6])
                gamma += op.gamma()

            print "Computed a " + der[0] + " gamma"

        return gamma
        

class Dispatcher(pb.Root):
    def __init__(self):
        self.two = None

    def remote_Portfolio(self,portfolio):
        op = SPortfolio(portfolio)
        print "returning a remote portfolio", op
        return op


if __name__ == '__main__':
    from twisted.internet import reactor
    rank = comm.Get_rank()
    print "hello from",rank
    reactor.listenTCP(8800+rank, pb.PBServerFactory(Dispatcher()))
    reactor.run()

    
