#include "OptionEuropean.h"

//Defines for statistics
//Normal PDF
#define NormZ(x) boost::math::pdf(normal,x)
//Normal CDF
#define NormQ(x) boost::math::cdf(normal,x)

//Signatures for utility functions

//Normal distribution as a global constant
boost::math::normal normal; //TODO: solve this.
// Utility functions

double d1(double S, double E, double tau, double r, double q, double sig){
  return (log(S/E)+(r-q+sig*sig/2)*tau)/(sig*sqrt(tau));
}

double d2(double S, double E, double tau, double r, double q, double sig){
  return (log(S/E)+(r-q-sig*sig/2)*tau)/(sig*sqrt(tau));
}

double put(double S, double E, double tau, double r, double q, double dc1,
           double dc2){
  return E*exp(-r*tau)*(1-NormQ(dc2))-S*exp(-q*tau)*(1-NormQ(dc1));
}

double call(double S, double E, double tau, double r, double q, double dc1,
            double dc2){
  return S*exp(-q*tau)*NormQ(dc1)-E*exp(-r*tau)*NormQ(dc2);
}

//Member functions

//Empty allocator.  Necessary to register it in the Factory
OptionPutEuropeanSingle::OptionPutEuropeanSingle()
{
  this->kind = "OptionPutEuropeanSingle";
}

OptionPutEuropeanSingle::OptionPutEuropeanSingle(double S, double E,
                                                 double tau, double r,
                                                 double q, double sig)
{
  this->E = E;
  this->tau = tau;
  this->r = r;
  this->pushAsset(1,S,q,sig);
  this->kind = "OptionPutEuropeanSingle";
}

OptionPutEuropeanSingle::~OptionPutEuropeanSingle(){}

double OptionPutEuropeanSingle::value()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return put(S,E,tau,r,q,dc1,dc2);
}

double OptionPutEuropeanSingle::delta()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return exp(-q*tau)*(NormQ(dc1)-1);
}

double OptionPutEuropeanSingle::gamma()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return exp(-q*tau)*NormZ(dc1)/(S*sig*sqrt(tau));
}

double OptionPutEuropeanSingle::theta()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return -q*exp(-q*tau)*S*NormQ(-dc1)+r*E*exp(-r*tau)*
      NormQ(-dc2)-(S*NormZ(dc1)*sig*exp(-q*tau))/
      (2*sqrt(tau));
}

double OptionPutEuropeanSingle::rho()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return -E*tau*exp(-r*tau)*NormQ(-dc2);
}


double OptionPutEuropeanSingle::vega()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return S*exp(-q*tau)*NormZ(dc1)*sqrt(tau);
}


/**********************************************************
            OptionCallEuropeanSingle
  *********************************************************/

// !Look at the comments below.

//Member functions

//Empty allocator.  Useful for indexing available derivatives
OptionCallEuropeanSingle::OptionCallEuropeanSingle()
{
  this->kind = "OptionCallEuropeanSingle";
}

OptionCallEuropeanSingle::OptionCallEuropeanSingle(double S, double E,
                                                   double tau, double r,
                                                   double q, double sig)
{
  this->E = E;
  this->tau = tau;
  this->r = r;
  this->pushAsset(1,S,q,sig);
  this->kind = "OptionCallEuropeanSingle";

}

OptionCallEuropeanSingle::~OptionCallEuropeanSingle(){}

double OptionCallEuropeanSingle::value()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return call(S,E,tau,r,q,dc1,dc2);
}

double OptionCallEuropeanSingle::delta()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return exp(-q*tau)*(NormQ(dc1));
}

double OptionCallEuropeanSingle::gamma()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return exp(-q*tau)*NormZ(dc1)/(S*sig*sqrt(tau));
}

double OptionCallEuropeanSingle::theta()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return q*exp(-q*tau)*S*NormQ(dc1)-r*E*exp(-r*tau)*NormQ(dc2)-
      (S*NormZ(dc1)*sig*exp(-q*tau))/(2*sqrt(tau));
}

double OptionCallEuropeanSingle::rho()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return E*tau*exp(-r*tau)*NormQ(dc2);
}

double OptionCallEuropeanSingle::vega()
{
  double S = this->assets->front().S;
  double E = this->E;
  double tau = this->tau;
  double r = this->r;
  double q = this->assets->front().q;
  double sig = this->assets->front().sig;
  double dc1 = d1(S,E,tau,r,q,sig);
  double dc2 = d2(S,E,tau,r,q,sig);
  return S*exp(-q*tau)*NormZ(dc1)*sqrt(tau);
}
