#!/usr/bin/env python

from distutils.core import setup, Extension
import os
import sys


cls = [
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    'Programming Language :: C++',
    'Programming Language :: Python :: 2.6',
    'Topic :: Office/Business :: Financial',
    'Topic :: Scientific/Engineering :: Mathematics'
]

includedirs = [os.path.join(os.curdir,'ctlib','lib')]
librarydirs = list()
cflags = list()
ldflags = list()

if sys.platform =="win32":
    includedirs.append(os.path.join(os.path.expandvars('$ProgramFiles'),
	    	'boost','boost_1_39'))
    librarydirs.append(os.path.join(os.path.expandvars('$ProgramFiles'),
	    	'boost','boost_1_39','lib'))
    cflags = ['/EHsc','/openmp']
    ldflags = ['/openmp']

if sys.platform == "linux2":
    cflags = ['-fopenmp']
    ldflags = ['-fopenmp']


setup(name='ctlib',
      version='0.1.1',
      description='Library for parallel computaion of derivatives properties',
      author='Guillem Borrell',
      author_email='guillemborrell@gmail.com',
      py_modules=['ctlib.lib.wrapper','ctlib.__init__','ctlib.lib.__init__'],
      ext_modules=[Extension(
            'ctlib.lib._wrapper',['ctlib/lib/wrapper.i',
                                  'ctlib/lib/Derivative.cpp',
                                  'ctlib/lib/OptionEuropean.cpp',
                                  'ctlib/lib/OptionAmerican.cpp',],
            include_dirs=includedirs,
            library_dirs=librarydirs,
            extra_compile_args=cflags,
            extra_link_args=ldflags,
            swig_opts=['-c++','-threads'])],
      scripts=['ctlib/ctclient.py','ctlib/ctserver_serial.py',
               'ctlib/ctserver.py'],
      classifiers=cls,
      long_description="""
ctlib, for Continuous Trading, is a small project intended as a small test case.

Its goal is to integrate several technologies and to apply them in a single supercomputing application:

* Python scripting

* Extension modules and Swig

* MPI

* OpenMP

* Grid applications with twisted

* Multiplatform development.

The application computes the value of some derivatives (very few) using a client server paraigm with Twisted.  You need both the server and the client to value derivatives.
"""
      #install_requires = ['mpi4py']
      )
